@@ -4,19 +4,13 @@
 from __future__ import absolute_import
 
 import numpy as np
-import warnings
-try:
-    import pandas as pd
-    _IS_PANDAS_INSTALLED = True
-except ImportError:
-    _IS_PANDAS_INSTALLED = False
 
 from .basic import Dataset, LightGBMError
 from .compat import (SKLEARN_INSTALLED, _LGBMClassifierBase,
                      LGBMNotFittedError, _LGBMLabelEncoder, _LGBMModelBase,
                      _LGBMRegressorBase, _LGBMCheckXY, _LGBMCheckArray, _LGBMCheckConsistentLength,
                      _LGBMCheckClassificationTargets, _LGBMComputeSampleWeight,
-                     argc_, range_, LGBMDeprecationWarning)
+                     argc_, range_, DataFrame)
 from .engine import train
 
 
@@ -310,7 +304,7 @@ class LGBMModel(_LGBMModelBase):
             Weights of training data.
         init_score : array-like of shape = [n_samples] or None, optional (default=None)
             Init score of training data.
-        group : array-like of shape = [n_samples] or None, optional (default=None)
+        group : array-like or None, optional (default=None)
             Group data of training data.
         eval_set : list or None, optional (default=None)
             A list of (X, y) tuple pairs to use as a validation sets for early-stopping.
@@ -327,8 +321,10 @@ class LGBMModel(_LGBMModelBase):
         eval_metric : string, list of strings, callable or None, optional (default=None)
             If string, it should be a built-in evaluation metric to use.
             If callable, it should be a custom evaluation metric, see note for more details.
+            In either case, the ``metric`` from the model parameters will be evaluated and used as well.
         early_stopping_rounds : int or None, optional (default=None)
             Activates early stopping. The model will train until the validation score stops improving.
+            If there's more than one, will check all of them.
             Validation error needs to decrease at least every ``early_stopping_rounds`` round(s)
             to continue training.
         verbose : bool, optional (default=True)
@@ -414,7 +410,7 @@ class LGBMModel(_LGBMModelBase):
             feval = None
             params['metric'] = eval_metric
 
-        if not _IS_PANDAS_INSTALLED or not isinstance(X, pd.DataFrame):
+        if not isinstance(X, DataFrame):
             X, y = _LGBMCheckXY(X, y, accept_sparse=True, force_all_finite=False, ensure_min_samples=2)
             _LGBMCheckConsistentLength(X, y, sample_weight)
 
@@ -504,7 +500,7 @@ class LGBMModel(_LGBMModelBase):
         """
         if self._n_features is None:
             raise LGBMNotFittedError("Estimator not fitted, call `fit` before exploiting the model.")
-        if not _IS_PANDAS_INSTALLED or not isinstance(X, pd.DataFrame):
+        if not isinstance(X, DataFrame):
             X = _LGBMCheckArray(X, accept_sparse=True, force_all_finite=False)
         n_features = X.shape[1]
         if self._n_features != n_features:
@@ -531,7 +527,7 @@ class LGBMModel(_LGBMModelBase):
         """
         if self._n_features is None:
             raise LGBMNotFittedError("Estimator not fitted, call `fit` before exploiting the model.")
-        if not _IS_PANDAS_INSTALLED or not isinstance(X, pd.DataFrame):
+        if not isinstance(X, DataFrame):
             X = _LGBMCheckArray(X, accept_sparse=True, force_all_finite=False)
         n_features = X.shape[1]
         if self._n_features != n_features:
@@ -708,7 +704,7 @@ class LGBMClassifier(LGBMModel, _LGBMClassifierBase):
         """
         if self._n_features is None:
             raise LGBMNotFittedError("Estimator not fitted, call `fit` before exploiting the model.")
-        if not _IS_PANDAS_INSTALLED or not isinstance(X, pd.DataFrame):
+        if not isinstance(X, DataFrame):
             X = _LGBMCheckArray(X, accept_sparse=True, force_all_finite=False)
         n_features = X.shape[1]
         if self._n_features != n_features:
